/* pvfs_mount.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

 */

static char pvfs_mount_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_mount.c,v 1.3 1999/10/15 22:05:57 rbross Exp $";

/* This file contains PVFS library call for mount.   		*/
#include <lib.h>
#include <meta.h>

extern int errno;
extern int pvfs_checks_disabled;

int pvfs_mount(const char* specialfile, char* dir, const char*
	filesystemtype, unsigned long rwflag, const void* data)
{
	int s, i, len; 
	mreq req;
	mack ack;
#ifdef __USE_NFS__
	dmeta dotdir;
#else
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;
#endif

	if (pvfs_checks_disabled) return(syscall(SYS_mount, specialfile,
		dir, filesystemtype, rwflag, data));

#ifdef __USE_NFS__
	len = strlen(dir);
	if (dir[len-1] != '/') dir[len] = '/';
#endif
	/* Check to see if .pvfsdir exists in directory  	*/
#ifdef __USE_NFS__
	if ((i = check_pvfs(dir)) < 1) {
#else
	if ((i = pvfs_detect(dir, &fn, &saddr, &fs_ino)) < 1) {
#endif
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return (syscall(SYS_mount, specialfile, dir, filesystemtype,
				rwflag, data));
	}
#ifdef __USE_NFS__
	if ((i = get_dmeta(dir, &dotdir)) < 0) {
		PERROR("Error getting file");
		return(-1);
	}
#endif
	/* Prepare request for file system  */
#ifdef __USE_NFS__
	req.uid = getuid();
	req.gid = getgid();
	req.dsize = strlen(dotdir.rd_path);
#else
	req.dsize = strlen(fn);
#endif
	req.type = MGR_MOUNT;
	req.req.mount.fsname = 0;

	/* send request to manager */
#ifdef __USE_NFS__
	if ((s = send_mreq_dmeta(&dotdir, &req, dotdir.rd_path, &ack)) < 0) {
#else
	if ((s = send_mreq_saddr(saddr, &req, fn, &ack)) < 0) {
#endif
		PERROR("send_mreq");
		return(-1);
	}
	return (0);
}
